@@ -11,8 +11,11 @@
 #include "editgroupdialog.h"
 
 #include "commands/detailscommand.h"
+#include "utils/gui-helper.h"
 #include "view/keytreeview.h"
+#include <settings.h>
 
+#include <Libkleo/DefaultKeyFilter>
 #include <Libkleo/KeyListModel>
 
 #include <KConfigGroup>
@@ -23,6 +26,7 @@
 #include <KStandardGuiItem>
 
 #include <QDialogButtonBox>
+#include <QGroupBox>
 #include <QHBoxLayout>
 #include <QItemSelectionModel>
 #include <QLabel>
@@ -40,6 +44,16 @@ using namespace GpgME;
 
 Q_DECLARE_METATYPE(GpgME::Key)
 
+namespace
+{
+auto createOpenPGPOnlyKeyFilter()
+{
+    auto filter = std::make_shared<DefaultKeyFilter>();
+    filter->setIsOpenPGP(DefaultKeyFilter::Set);
+    return filter;
+}
+}
+
 class EditGroupDialog::Private
 {
     friend class ::Kleo::Dialogs::EditGroupDialog;
@@ -62,45 +76,70 @@ public:
     {
         auto mainLayout = new QVBoxLayout(q);
 
+        {
         auto groupNameLayout = new QHBoxLayout();
-        groupNameLayout->addWidget(new QLabel(i18nc("Name of a group of keys", "Name:")));
-        ui.groupNameEdit = new QLineEdit();
+        auto label = new QLabel(i18nc("Name of a group of keys", "Name:"), q);
+        groupNameLayout->addWidget(label);
+        ui.groupNameEdit = new QLineEdit(q);
+        label->setBuddy(ui.groupNameEdit);
         groupNameLayout->addWidget(ui.groupNameEdit);
         mainLayout->addLayout(groupNameLayout);
+        }
 
-        mainLayout->addWidget(new KSeparator(Qt::Horizontal));
+        mainLayout->addWidget(new KSeparator(Qt::Horizontal, q));
 
-        auto centerLayout = new QVBoxLayout();
+        auto centerLayout = new QVBoxLayout;
 
-        auto availableKeysLayout = new QVBoxLayout();
-        availableKeysLayout->addWidget(new QLabel(i18n("Available keys:")));
+        auto availableKeysGroupBox = new QGroupBox{i18nc("@title", "Available Keys"), q};
+        availableKeysGroupBox->setFlat(true);
+        auto availableKeysLayout = new QVBoxLayout{availableKeysGroupBox};
 
-        ui.availableKeysFilter = new QLineEdit();
-        ui.availableKeysFilter->setClearButtonEnabled(true);
-        ui.availableKeysFilter->setPlaceholderText(i18nc("Placeholder text", "Search..."));
-        availableKeysLayout->addWidget(ui.availableKeysFilter);
+        {
+            auto hbox = new QHBoxLayout;
+            auto label = new QLabel{i18nc("@label", "Search:")};
+            label->setAccessibleName(i18nc("@label", "Search available keys"));
+            label->setToolTip(i18nc("@info:tooltip", "Search the list of available keys for keys matching the search term."));
+            hbox->addWidget(label);
+
+            ui.availableKeysFilter = new QLineEdit(q);
+            ui.availableKeysFilter->setClearButtonEnabled(true);
+            ui.availableKeysFilter->setAccessibleName(i18nc("@label", "Search available keys"));
+            ui.availableKeysFilter->setToolTip(i18nc("@info:tooltip", "Search the list of available keys for keys matching the search term."));
+            ui.availableKeysFilter->setPlaceholderText(i18nc("@info::placeholder", "Enter search term"));
+            ui.availableKeysFilter->setCursorPosition(0); // prevent emission of accessible text cursor event before accessible focus event
+            label->setBuddy(ui.availableKeysFilter);
+            hbox->addWidget(ui.availableKeysFilter, 1);
+
+            availableKeysLayout->addLayout(hbox);
+        }
 
         availableKeysModel = AbstractKeyListModel::createFlatKeyListModel(q);
         availableKeysModel->useKeyCache(true, KeyList::AllKeys);
         ui.availableKeysList = new KeyTreeView(q);
+        ui.availableKeysList->view()->setAccessibleName(i18n("available keys"));
         ui.availableKeysList->view()->setRootIsDecorated(false);
         ui.availableKeysList->setFlatModel(availableKeysModel);
         ui.availableKeysList->setHierarchicalView(false);
+        if (!Settings{}.cmsEnabled()) {
+            ui.availableKeysList->setKeyFilter(createOpenPGPOnlyKeyFilter());
+        }
         availableKeysLayout->addWidget(ui.availableKeysList, /*stretch=*/ 1);
 
-        centerLayout->addLayout(availableKeysLayout, /*stretch=*/ 1);
+        centerLayout->addWidget(availableKeysGroupBox, /*stretch=*/ 1);
 
-        auto buttonsLayout = new QHBoxLayout();
+        auto buttonsLayout = new QHBoxLayout;
         buttonsLayout->addStretch(1);
 
-        auto addButton = new QPushButton();
+        auto addButton = new QPushButton(q);
         addButton->setIcon(QIcon::fromTheme(QStringLiteral("arrow-down")));
+        addButton->setAccessibleName(i18nc("@action:button", "Add Selected Keys"));
         addButton->setToolTip(i18n("Add the selected keys to the group"));
         addButton->setEnabled(false);
         buttonsLayout->addWidget(addButton);
 
-        auto removeButton = new QPushButton();
+        auto removeButton = new QPushButton(q);
         removeButton->setIcon(QIcon::fromTheme(QStringLiteral("arrow-up")));
+        removeButton->setAccessibleName(i18nc("@action:button", "Remove Selected Keys"));
         removeButton->setToolTip(i18n("Remove the selected keys from the group"));
         removeButton->setEnabled(false);
         buttonsLayout->addWidget(removeButton);
@@ -109,34 +148,53 @@ public:
 
         centerLayout->addLayout(buttonsLayout);
 
-        auto groupKeysLayout = new QVBoxLayout();
-        groupKeysLayout->addWidget(new QLabel(i18n("Group keys:")));
+        auto groupKeysGroupBox = new QGroupBox{i18nc("@title", "Group Keys"), q};
+        groupKeysGroupBox->setFlat(true);
+        auto groupKeysLayout = new QVBoxLayout{groupKeysGroupBox};
 
-        ui.groupKeysFilter = new QLineEdit();
-        ui.groupKeysFilter->setClearButtonEnabled(true);
-        ui.groupKeysFilter->setPlaceholderText(i18nc("Placeholder text", "Search..."));
-        groupKeysLayout->addWidget(ui.groupKeysFilter);
+        {
+            auto hbox = new QHBoxLayout;
+            auto label = new QLabel{i18nc("@label", "Search:")};
+            label->setAccessibleName(i18nc("@label", "Search group keys"));
+            label->setToolTip(i18nc("@info:tooltip", "Search the list of group keys for keys matching the search term."));
+            hbox->addWidget(label);
+
+            ui.groupKeysFilter = new QLineEdit(q);
+            ui.groupKeysFilter->setClearButtonEnabled(true);
+            ui.groupKeysFilter->setAccessibleName(i18nc("@label", "Search group keys"));
+            ui.groupKeysFilter->setToolTip(i18nc("@info:tooltip", "Search the list of group keys for keys matching the search term."));
+            ui.groupKeysFilter->setPlaceholderText(i18nc("@info::placeholder", "Enter search term"));
+            ui.groupKeysFilter->setCursorPosition(0); // prevent emission of accessible text cursor event before accessible focus event
+            label->setBuddy(ui.groupKeysFilter);
+            hbox->addWidget(ui.groupKeysFilter, 1);
+
+            groupKeysLayout->addLayout(hbox);
+        }
 
         groupKeysModel = AbstractKeyListModel::createFlatKeyListModel(q);
         ui.groupKeysList = new KeyTreeView(q);
+        ui.groupKeysList->view()->setAccessibleName(i18n("group keys"));
         ui.groupKeysList->view()->setRootIsDecorated(false);
         ui.groupKeysList->setFlatModel(groupKeysModel);
         ui.groupKeysList->setHierarchicalView(false);
         groupKeysLayout->addWidget(ui.groupKeysList, /*stretch=*/ 1);
 
-        centerLayout->addLayout(groupKeysLayout, /*stretch=*/ 1);
+        centerLayout->addWidget(groupKeysGroupBox, /*stretch=*/ 1);
 
         mainLayout->addLayout(centerLayout);
 
-        mainLayout->addWidget(new KSeparator(Qt::Horizontal));
+        mainLayout->addWidget(new KSeparator(Qt::Horizontal, q));
 
-        ui.buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
+        ui.buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel, q);
         QPushButton *okButton = ui.buttonBox->button(QDialogButtonBox::Ok);
         KGuiItem::assign(okButton, KStandardGuiItem::ok());
         KGuiItem::assign(ui.buttonBox->button(QDialogButtonBox::Cancel), KStandardGuiItem::cancel());
         okButton->setEnabled(false);
         mainLayout->addWidget(ui.buttonBox);
 
+        // prevent accidental closing of dialog when pressing Enter while a search field has focus
+        Kleo::unsetAutoDefaultButtons(q);
+
         connect(ui.groupNameEdit, &QLineEdit::textChanged,
                 q, [okButton] (const QString &text) {
                     okButton->setEnabled(!text.trimmed().isEmpty());
@@ -213,7 +271,7 @@ private:
         }
         const auto key = index.model()->data(index, KeyList::KeyRole).value<GpgME::Key>();
         if (!key.isNull()) {
-            auto cmd = new DetailsCommand(key, nullptr);
+            auto cmd = new DetailsCommand(key);
             cmd->setParentWidget(q);
             cmd->start();
         }
@@ -289,3 +347,11 @@ std::vector<Key> EditGroupDialog::groupKeys() const
     }
     return keys;
 }
+
+void EditGroupDialog::showEvent(QShowEvent *event)
+{
+    QDialog::showEvent(event);
+
+    // prevent accidental closing of dialog when pressing Enter while a search field has focus
+    Kleo::unsetDefaultButtons(d->ui.buttonBox);
+}
